<?php

namespace SM\Service;

use SM\Factory\ModeloDocumentoCdaFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use SM\Service\FileServerService;
use SM\Util\Utilities;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class ModeloDocumentoCdaService
{

    protected $ModeloDocumentoCdaFactory;
    protected $Environment;
    protected $newCode;
    protected $fileServerService;
    protected $fileServerUrl;
    protected $connection;

    public function __construct(EntityManager $entityManager, ModeloDocumentoCdaFactory $ModeloDocumentoCdaFactory, Environment $Environment, NewCode $newCode, FileServerService $fileServerService, $fileServerUrl)
    {
        $this->ModeloDocumentoCdaFactory = $ModeloDocumentoCdaFactory;
        $this->connection             = $entityManager->getConnection();
        $this->Environment            = $Environment;
        $this->newCode                = $newCode;
        $this->fileServerService      = $fileServerService;
        $this->fileServerUrl          = $fileServerUrl;
    }
 
    public function getInformacoesModeloDocumentoCda()
    {
        return $this->ModeloDocumentoCdaFactory->getInformacoesModeloDocumentoCda();
    }

    public function getTabAnexoCda($params)
    {
        $paramsaux = array(
            "NRMODELODOCCDA"   => $params['NRMODELODOCCDA']
        );
        return $this->ModeloDocumentoCdaFactory->getTabAnexoCda($paramsaux);
    }

    public function getModeloDocumentoPadraoCda($params)
    {
        $paramsaux = array(
            "CDIDIOM"   => $params['CDIDIOM']
        );
        return $this->ModeloDocumentoCdaFactory->getModeloDocumentoPadraoCda($paramsaux);
    }
    

    public function inserirModeloDocumentoCda($parametros)
    {
        $nrorg = $this->Environment->getNrorg();
        $cdoperador = $this->Environment->getCdOperador();
        $nrmodelodoccda = $this->newCode->getCodeByTableName("MODELODOCCDA", $nrorg, 1, 10, true);
            $params = array(
                "NRMODELODOCCDA" => $nrmodelodoccda,
                "DSMODELODOCORCA" => $parametros['DSMODELODOCORCA'],
                "IDATIVO" => $parametros['IDATIVO'],
                "IDDOCPADRAO" => $parametros['IDDOCPADRAO'],
                "NRORG"          => $nrorg,
                "NRORGINCLUSAO"  => $nrorg,
                "CDOPERINCLUSAO" => $cdoperador,
            );
            
            $this->ModeloDocumentoCdaFactory->inserirModeloDocumentoCda($params);
            
            if ($parametros["IDDOCPADRAO"] == "S") {
                $dataParam = array(
                    "IDDOCPADRAO" => "N",
                    "NRMODELODOCCDA"  => $nrmodelodoccda,
                );
                $this->ModeloDocumentoCdaFactory->atualizarModeloDocumentoPadrao($dataParam);
            }
    }

    public function inserirModeloDocumentoIdiomaCda($parametros)
    {
        $nrorg = $this->Environment->getNrorg();
        $cdoperador = $this->Environment->getCdOperador();
        $this->connection->beginTransaction();
        
        $paramsauxdoc = array(
            "NRMODELODOCCDA" => $parametros['NRMODELODOCCDA'],
            "NRMODELODOCCDAIDI" => $parametros['NRMODELODOCCDAIDI'],
        );

        $paramsauxdocidioma = array(
            "NRMODELODOCCDA"    => $parametros['NRMODELODOCCDA'],
            "CDIDIOM"           => $parametros['CDIDIOM'],
            "NRMODELODOCCDAIDI" => $parametros['NRMODELODOCCDAIDI'],
        );

        $valor = $this->ModeloDocumentoCdaFactory->validaDocIdioma($paramsauxdocidioma);

        if ($valor[0]["QTD"] > 0) {
            throw new \Exception("EXISTE");
        }
        
        try {
            if (count($parametros['ARQUIVO']) > 0) {
                $dados = $this->fileServerService->uploadFiles($parametros['ARQUIVO'], 'anexo_modelo_documento_cda');
                $dados[0]["size"] = $dados[0]["size"]/1024;
            } else {
                $dados = [[
                    "path" => $parametros["DSCAMINHOARQ"],
                    "size" => $parametros["VRTAMANHO"],
                    "name" => $parametros["NMARQUIVO"],
                    "type" => $parametros["DSTIPO"],
                ]];
            }

            if ($parametros["IDPADRAO"] == "S") {
                $params = array(
                    "NRMODELODOCCDA"    => $parametros['NRMODELODOCCDA'],
                    "NMARQUIVO"         => $dados[0]["name"],
                    "DSCAMINHOARQ"      => $dados[0]["path"],
                    "DSTIPO"            => $dados[0]["type"],
                    "VRTAMANHO"         => $dados[0]["size"],
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERULTATU"      => $cdoperador
                );

                $this->ModeloDocumentoCdaFactory->atualizarModeloDocumentoCda($params);
            }

            if ($parametros['NRMODELODOCCDAIDI'] == null) {
                $nrmodelodoccdaidi = $this->newCode->getCodeByTableName("MODELODOCCDAIDI", $nrorg, 1, 10, true);
                $params = array(
                    "NRMODELODOCCDAIDI" => $nrmodelodoccdaidi,
                    "NRMODELODOCCDA"    => $parametros['NRMODELODOCCDA'],
                    "CDIDIOM"           => $parametros['CDIDIOM'],
                    "IDATIVO"           => $parametros['IDATIVO'],
                    "NMARQUIVO"         => $dados[0]['name'],
                    "DSCAMINHOARQ"      => $dados[0]['path'],
                    "DSTIPO"            => $dados[0]['type'],
                    "VRTAMANHO"         => $dados[0]['size'],
                    "NRORG"             => $nrorg,
                    "NRORGINCLUSAO"     => $nrorg,
                    "CDOPERINCLUSAO"    => $cdoperador,
                );
                
                $this->ModeloDocumentoCdaFactory->inserirModeloDocumentoIdiomaCda($params);
            } else {
                $params = array(
                    "NRMODELODOCCDAIDI" => $parametros['NRMODELODOCCDAIDI'],
                    "NRMODELODOCCDA"    => $parametros['NRMODELODOCCDA'],
                    "CDIDIOM"           => $parametros['CDIDIOM'],
                    "IDATIVO"           => $parametros['IDATIVO'],
                    "NMARQUIVO"         => $dados[0]["name"],
                    "DSCAMINHOARQ"      => $dados[0]["path"],
                    "DSTIPO"            => $dados[0]["type"],
                    "VRTAMANHO"         => $dados[0]["size"],
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERULTATU"      => $cdoperador
                );
                
                $this->ModeloDocumentoCdaFactory->atualizarModeloDocumentoIdiomaCda($params);
            }
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveTabModeloDocumentoCda($parametros)
    {
        $nrorg = $this->Environment->getNrorg();
        $cdoperador = $this->Environment->getCdOperador();
        $this->connection->beginTransaction();
        try {
            $params = array(
                "NRMODELODOCCDA"    => $parametros['NRMODELODOCCDA'],
                "DSMODELODOCORCA"   => $parametros['DSMODELODOCORCA'],
                "IDATIVO"           => $parametros['IDATIVO'],
                "IDDOCPADRAO"       => $parametros['IDDOCPADRAO'],
                "NRORGULTATU"       => $nrorg,
                "CDOPERULTATU"      => $cdoperador,
            );
            $this->ModeloDocumentoCdaFactory->saveTabModeloDocumentoCda($params);
            if ($parametros["IDDOCPADRAO"] == "S") {
                $dataParam = array(
                    "IDDOCPADRAO" => "N",
                    "NRMODELODOCCDA"  => $parametros["NRMODELODOCCDA"],
                );
                $this->ModeloDocumentoCdaFactory->atualizarModeloDocumentoPadrao($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deletarAnexoCda($params)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($params as $anexo) {
                $dataParam = array(
                    "NRMODELODOCCDAIDI" => $anexo["NRMODELODOCCDAIDI"],
                );
                if (!empty($anexo['DSCAMINHOARQ'])) {
                    $this->fileServerService->deleteFile($anexo['DSCAMINHOARQ']);
                }
                if ($dataParam["NRMODELODOCCDAIDI"] !== null) {
                    $this->ModeloDocumentoCdaFactory->deletarAnexoCda($dataParam);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deletarModeloDocumentoCda($parametros)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($parametros['NRS'] as $nr) {
                $param = array(
                    "NRMODELODOCCDA" => $nr
                );

                $anexos = $this->getTabAnexoCda($param);
                $this->deletarAnexoCda($anexos["data"]);
                $this->ModeloDocumentoCdaFactory->deletarModeloDocumentoCda($param);
            }
        
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function limparAnexoModeloDocumentoCda($parametros)
    {
        $row = $parametros["row"];

        $param = array(
            "NRMODELODOCCDAIDI" => $row["NRMODELODOCCDAIDI"]
        );

        $this->fileServerService->deleteFile($row["DSCAMINHOARQ"]);
        $this->ModeloDocumentoCdaFactory->limparAnexoModeloDocumentoCda($param);
    }
    
    public function downloadAnexoModeloDocumentoCda($nrmodelodoccdaidi)
    {
        $param = array(
            "NRMODELODOCCDAIDI" => $nrmodelodoccdaidi
        );
        $dados = $this->ModeloDocumentoCdaFactory->getDocumentoCda($param);

        $url = $this->fileServerUrl."/files/".$dados[0]['DSCAMINHOARQ'];
        $nomeArquivo = $dados[0]['NMARQUIVO'];
        $tipo = $dados[0]['DSTIPO'];
        $tamanho = $dados[0]['VRTAMANHO']*1024;

        $file = @file_get_contents($url);
        if ($file) {
            Utilities::downloadArquivoFileServer($url, $nomeArquivo, $tipo, $tamanho);
        } else {
            echo "Arquivo não encontrado.";
            exit;
        }
    }
}
