<?php
namespace SM\Service;

use SM\Factory\Sm00054002ReducaoQuantidadeMedidaPerfilAcessoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class Sm00054002ReducaoQuantidadeMedidaPerfilAcessoService
{
    protected $connection;
    protected $reducaoQuantidadeMedidaPerfilAcessoFactory;
    protected $environment;
    protected $newCode;

    public function __construct(Sm00054002ReducaoQuantidadeMedidaPerfilAcessoFactory $reducaoQuantidadeMedidaPerfilAcessoFactory, Environment $environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->reducaoQuantidadeMedidaPerfilAcessoFactory            = $reducaoQuantidadeMedidaPerfilAcessoFactory;
        $this->environment               = $environment;
        $this->newCode                   = $newCode;
        $this->connection                = $entityManager->getConnection();
    }

    public function getReducaoQuantidadeMedidaPerfilAcesso()
    {
        return $this->reducaoQuantidadeMedidaPerfilAcessoFactory->getReducaoQuantidadeMedidaPerfilAcesso();
    }
    
    public function inserirReducaoQuantidadeMedidaPerfilAcesso($parametros)
    {
      
        $cdoperador = $this->environment->getCdOperador();
        $NRORG = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        $paramsaux = array(
          "NRPERFILACESSO" => $parametros[0]['NRPERFILACESSO'],
          "NRALTERAQTMEDPERF" => $parametros[0]['NRALTERAQTMEDPERF']
        );

        $dados = $this->reducaoQuantidadeMedidaPerfilAcessoFactory->validaReducaoPerfilAcesso($paramsaux);

        try {
            foreach ($parametros as $chave => $perfil) {
                if ($perfil["NRALTERAQTMEDPERF"] == 0) {
                    $NRALTERAQTMEDPERF = $this->newCode->getCodeByTableName("FCT_ALTERAQTMEDPERFACESS", $NRORG, 1, 10, true);
                    $dataParam = array(
                    "NRALTERAQTMEDPERF" => $NRALTERAQTMEDPERF,
                    "NRPERFILACESSO"   => $perfil["NRPERFILACESSO"],
                    "NRORG"              => $NRORG,
                    "IDATIVO"       => "S",
                    "NRORGINCLUSAO"      => $NRORG,
                    "CDOPERINCLUSAO"     => $cdoperador,
                    );
                    if ($dados[0]["QTD"] > 0) {
                        throw new \Exception("EXISTE");
                    } else {
                        $this->reducaoQuantidadeMedidaPerfilAcessoFactory->inserirReducaoQuantidadeMedidaPerfilAcesso($dataParam);
                    }
                } else {
                    $dataParam = array(
                
                    "NRPERFILACESSO"   => $perfil["NRPERFILACESSO"],
                    "NRORG"              => $NRORG,
                    "IDATIVO"       => "S",
                    "NRORGINCLUSAO"      => $NRORG,
                    "CDOPERINCLUSAO"     => $cdoperador,
                    "NRALTERAQTMEDPERF" => $perfil["NRALTERAQTMEDPERF"],
                    );
                    if ($dados[0]["QTD"] > 0) {
                        throw new \Exception("EXISTE");
                    } else {
                        $this->reducaoQuantidadeMedidaPerfilAcessoFactory->atualizarReducaoQuantidadeMedidaPerfilAcesso($dataParam);
                    }
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
      
    public function deletarReducaoQuantidadeMedidaPerfilAcesso($parametros)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($parametros as $chave => $perfil) {
                $dataParam = array( "NRALTERAQTMEDPERF" => $perfil["NRALTERAQTMEDPERF"],);
                $this->reducaoQuantidadeMedidaPerfilAcessoFactory->deletarReducaoQuantidadeMedidaPerfilAcesso($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
