import { Utilities } from '@/util/Utilities';
import {
    DialogService,
    Form,
    LoadingService, Modal, ModalService, Select, SelectMultiple, Tabs, TextInput, Textarea,
} from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class SmAgrupamentoEscopoController {
    public visibilidade: boolean = true;

    public currentRow: any = {};

    public nomeAgrup: String = '';

    public modalAdicionar!: Modal;

    public modalEscopo!: Modal;

    get visibilidadeDoFrame() {
        return !this.visibilidade;
    }

    public async criarModal() {
        this.modalAdicionar = await ModalService.createFromJson('sm_agrupamento_escopo/modal/sm_adicionar_agrupamento', true);
        this.modalEscopo = await ModalService.createFromJson('sm_agrupamento_escopo/modal/sm_modal_adicionar_escopo', true);
    }

    public destruirModal() {
        this.modalAdicionar?.destroy();
        this.modalEscopo?.destroy();
    }

    public abrirModal() {
        this.modalAdicionar.show();
    }

    public fecharModal() {
        this.modalAdicionar.hide();
    }

    public expandir({ component }: any) {
        const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
        form.isVisible = !form.isVisible;

        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    public salvarAgrupamento() {
        const form = Metadata.getInstance<Form>('form-novo-agrupamento');
        const form2 = Metadata.getInstance<Form>('form-info-faturamento');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-agrupamento-tipo-orcamento');
            
        if (Utilities.validateForm(form) && Utilities.validateForm(form2)) {
            const serie = Metadata.getInstance<Select>('form-CDSERIE');
            const filial = Metadata.getInstance<Select>('form-CDFILIAL');

            this.insertAgrupamento({
                NMAGRUPAMENTOTIPOORCA: Metadata.getInstance<TextInput>('form-nome-agrupamento').value,
                DSOBSAGRUPTIPOORCA: Metadata.getInstance<Textarea>('form-observacoes-agrupamento').value,
                IDTPFATURA: Metadata.getInstance<Select>('form-IDTPFATURA').value,
                CDFILIAL: filial.selectValue.CDFILIAL,
                CDEMPRESA: filial.selectValue.CDEMPRESA,
                CDTIPOOPER: Metadata.getInstance<Select>('form-CDTIPOOPER').value,
                IDNATUOPER: Metadata.getInstance<Select>('form-IDNATUOPER').value,
                CDSERIE: serie.selectValue.CDSERIE,
                CDINSCMUNI: serie.selectValue.CDINSCMUNI,
                CDINSCESTA: serie.selectValue.CDINSCESTA,
                IDMUNIPRESTSERV: Metadata.getInstance<Select>('form-IDMUNIPRESTSERV').value,
                IDTIPONATUOPER: Metadata.getInstance<Select>('form-IDTIPONATUOPER').value,
            }).then(async () => {
                grid.datasource.get();
                this.fecharModal();
            });
        }
    }

    public editarAgrupamento() {
        const form = Metadata.getInstance<Form>('form2');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-agrupamento-tipo-orcamento');

        if (Utilities.validateForm(form)) {
            const serie = Metadata.getInstance<Select>('form2-CDSERIE');
            const filial = Metadata.getInstance<Select>('form2-CDFILIAL');

            this.insertAgrupamento({
                NRAGRUPAMENTOTIPOORCA: Metadata.getInstance<TextInput>('form2-NRAGRUPAMENTOTIPOORCA').value,
                NMAGRUPAMENTOTIPOORCA: Metadata.getInstance<TextInput>('form2-NMAGRUPAMENTOTIPOORCA').value,
                DSOBSAGRUPTIPOORCA: Metadata.getInstance<Textarea>('form2-DSOBSAGRUPTIPOORCA').value,
                IDTPFATURA: Metadata.getInstance<Select>('form2-IDTPFATURA').value,
                CDFILIAL: filial.selectValue.CDFILIAL,
                CDEMPRESA: filial.selectValue.CDEMPRESA,
                CDTIPOOPER: Metadata.getInstance<Select>('form2-CDTIPOOPER').value,
                IDNATUOPER: Metadata.getInstance<Select>('form2-IDNATUOPER').value,
                CDSERIE: serie.selectValue.CDSERIE,
                CDINSCMUNI: serie.selectValue.CDINSCMUNI,
                CDINSCESTA: serie.selectValue.CDINSCESTA,
                IDMUNIPRESTSERV: Metadata.getInstance<Select>('form2-IDMUNIPRESTSERV').value,
                IDTIPONATUOPER: Metadata.getInstance<Select>('form2-IDTIPONATUOPER').value,
            }).then(async () => {
                grid.datasource.get();
            });
        }
    }

    public async insertAgrupamento(parametros: any) {
        LoadingService.show();
        return Http.post('/insertAgrupamentoTipoOrca', parametros).then(() => {
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    /* eslint-disable */
    public canEditRow({ row }: any) {
        return row['__added_row'] === true;
    }
    /* eslint-enable */

    public rowClick({ row }: IEventParam<TekGrid>) {
        this.visibilidade = false;
        this.currentRow = row;
        this.nomeAgrup = row.NMAGRUPAMENTOTIPOORCA;
        this.recarregarTabs();
    }

    public voltarPaginaInicial() {
        this.visibilidade = true;
    }

    public recarregarTabs() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabs-agrupamento');
        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab-dadosGerais') {
            this.montarInformacoesDadosGerais();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab-tipo-orcamento') {
            setTimeout(() => {
                const grid = Metadata.getInstance<TekGrid>('tek-grid-tipo-orcamento');
                this.setFiltroGridEscopos({ component: grid });
            }, 1000);
        }
    }

    public montarInformacoesDadosGerais() {
        LoadingService.show();
        Metadata.getInstance<TextInput>('form2-NRAGRUPAMENTOTIPOORCA').value = this.currentRow.NRAGRUPAMENTOTIPOORCA;
        Metadata.getInstance<TextInput>('form2-NMAGRUPAMENTOTIPOORCA').value = this.currentRow.NMAGRUPAMENTOTIPOORCA;
        Metadata.getInstance<Textarea>('form2-DSOBSAGRUPTIPOORCA').value = this.currentRow.DSOBSAGRUPTIPOORCA;
        Metadata.getInstance<Select>('form2-IDTPFATURA').value = this.currentRow.IDTPFATURA;
        Metadata.getInstance<Select>('form2-CDFILIAL').value = this.currentRow.CDFILIAL;
        Metadata.getInstance<Select>('form2-CDTIPOOPER').value = this.currentRow.CDTIPOOPERFATURA;
        Metadata.getInstance<Select>('form2-IDNATUOPER').value = this.currentRow.IDNATUOPER;
        Metadata.getInstance<Select>('form2-IDMUNIPRESTSERV').value = this.currentRow.IDMUNIPRESTSERV;
        Metadata.getInstance<Select>('form2-IDTIPONATUOPER').value = this.currentRow.IDTIPONATUOPER;
        setTimeout(() => {
            Metadata.getInstance<Select>('form2-CDSERIE').value = this.currentRow.NRSERIE;
        }, 0);
        LoadingService.hide();
    }

    public changeNF({ component }: IEventParam<Select>) {
        const nf = Metadata.getInstance<Select>(`${component.name.split('-')[0]}-CDSERIE`);

        if (component.value) {
            nf.datasource.setFilter({
                CDEMPRESA: component.selectValue.CDEMPRESA,
            });
        }
    }

    public changeNatureza({ component }: IEventParam<Select>) {
        const natureza = Metadata.getInstance<Select>(`${component.name.split('-')[0]}-IDNATUOPER`);

        if (component.value === 'F') {
            natureza.disabled = false;
            natureza.addValidation('required');
        } else {
            natureza.disabled = true;
            natureza.removeValidation('required');
        }
    }

    public abrirModalEscopo() {
        this.modalEscopo.show();
        setTimeout(() => {
            const escopo = Metadata.getInstance<Select>('ESCOPO');
            escopo.value = null;
            escopo.datasource.setFilter({
                IDTPFATURA: this.currentRow.IDTPFATURA,
                CDFILIAL: this.currentRow.CDFILIAL,
                CDTIPOOPER: this.currentRow.CDTIPOOPERFATURA,
                IDNATUOPER: this.currentRow.IDNATUOPER,
                CDSERIE: this.currentRow.CDSERIE,
                IDMUNIPRESTSERV: this.currentRow.IDMUNIPRESTSERV,
                IDTIPONATUOPER: this.currentRow.IDTIPONATUOPER,
                CDINSCESTA: this.currentRow.CDINSCESTA,
                CDINSCMUNI: this.currentRow.CDINSCMUNI,
            });
        }, 500);
    }

    public fecharModalEscopo() {
        this.modalEscopo.hide();
    }

    public setFiltroGridEscopos({ component }: IEventParam<TekGrid>) {
        component.datasource.setFilter({
            NRAGRUPAMENTOTIPOORCA: this.currentRow.NRAGRUPAMENTOTIPOORCA,
        });
    }

    public salvarEscopo() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-tipo-orcamento');
        const form = Metadata.getInstance<Form>('form-novo-escopo');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/agruparEscopo', {
                NRTIPOORCAMENTO: Metadata.getInstance<SelectMultiple>('ESCOPO').value,
                NRAGRUPAMENTOTIPOORCA: this.currentRow.NRAGRUPAMENTOTIPOORCA,
            }).then(() => {
                Utilities.showSuccessMessage();
                grid.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
                this.fecharModalEscopo();
            });
        }
    }

    public showConfirmDeleteMessage({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-documento-proposta',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.excluirEscoposAgrupados();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public excluirEscoposAgrupados() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-tipo-orcamento');

        LoadingService.show();
        Http.post('/excluirEscoposAgrupados', {
            NRRELACAGRUPTIPOORCA: (grid.selectedRows.map((element: any) => element.NRRELACAGRUPTIPOORCA)),
        }).then(() => {
            Utilities.showSuccessMessage();
            grid.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
